@@ -1,11 +1,13 @@
-import pytest
+from codepack import Default, Delivery
 from codepack.interface import MongoDB
+from codepack.storage import MemoryMessenger
+import pytest
 import os
 import mongomock
-from codepack.config import Config, Default
+
+
 from shutil import rmtree
 from glob import glob
-from codepack.delivery import Delivery
 
 
 def mkdir(directory):
@@ -107,15 +109,7 @@ def init_default():
     Default.alias = None
     Default.config = None
     Default.instances = dict()
-
-
-@pytest.fixture(scope='function', autouse=False)
-def default():
-    os.environ['CODEPACK_ALIAS_PATH'] = 'codepack/config/default/alias.ini'
-    Default.init()
-    yield Default
-    Default.init()
-    os.environ.pop('CODEPACK_ALIAS_PATH', None)
+    MemoryMessenger.destroy()
 
 
 @pytest.fixture(scope='function', autouse=False)
@@ -124,9 +118,3 @@ def dummy_deliveries():
     obj2 = Delivery(id='obj2', serial_number='456', item='y')
     obj3 = Delivery(id='obj3', serial_number='789', item='y')
     yield [obj1, obj2, obj3]
-
-
-@pytest.fixture(scope='function', autouse=False)
-def conn_config():
-    config = Config(config_path='config/test_conn.ini')
-    yield config
